package com.progur.launcherassist;

import android.app.WallpaperManager;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.util.Base64;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.ByteArrayOutputStream;
import java.util.List;

import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * LauncherAssistPlugin
 */
public class LauncherAssistPlugin implements MethodCallHandler {

  private String wallpaperString = null;
  private PluginRegistry.Registrar registrar;

  public LauncherAssistPlugin(Registrar registrar) {
        this.registrar = registrar;
  }

  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "launcher_assist");
    channel.setMethodCallHandler(new LauncherAssistPlugin(registrar));
  }

  @Override
  public void onMethodCall(MethodCall methodCall, Result result) {
      if(methodCall.method.equals("getAllApps")) {
          getAllApps(result);
      } else if(methodCall.method.equals("launchApp")) {
          launchApp(methodCall.argument("packageName").toString());
      } else if(methodCall.method.equals("getWallpaper")) {
          getWallpaper(result);
      }
  }

  private void getWallpaper(MethodChannel.Result result) {
      if(wallpaperString != null) {
          result.success(wallpaperString);
          return;
      }

      WallpaperManager wallpaperManager = WallpaperManager.getInstance(registrar.context());
      Drawable wallpaperDrawable = wallpaperManager.getDrawable();
      if(wallpaperDrawable instanceof BitmapDrawable) {
          wallpaperString = encodeToBase64(((BitmapDrawable)wallpaperDrawable).getBitmap(),
                                                Bitmap.CompressFormat.JPEG, 100);
          result.success(wallpaperString);
      }
  }

  private void launchApp(String packageName) {
      Intent i = registrar.context().getPackageManager().getLaunchIntentForPackage(packageName);
      if(i != null)
        registrar.context().startActivity(i);
  }

  private void getAllApps(MethodChannel.Result result) {
      PackageManager manager = registrar.context().getPackageManager();
      List<ApplicationInfo> appsList = manager.getInstalledApplications(PackageManager.GET_META_DATA);

      try {
          JSONArray _output = new JSONArray();

          for (ApplicationInfo app : appsList) {
              if (manager.getLaunchIntentForPackage(app.packageName) != null) {

                  BitmapDrawable icon = (BitmapDrawable) app.loadIcon(manager);
                  String iconString = encodeToBase64(icon.getBitmap(),
                          Bitmap.CompressFormat.PNG, 100);

                  JSONObject current = new JSONObject();
                  current.put("label", app.loadLabel(manager).toString());
                  current.put("icon", iconString);
                  current.put("package", app.packageName);
                  _output.put(current);
              }
          }

          result.success(_output.toString(2));
      } catch(JSONException e) {
          Log.e("CANDY", "Couldn't create JSON object");
      }
  }

  public static String encodeToBase64(Bitmap image, Bitmap.CompressFormat compressFormat, int quality) {
    ByteArrayOutputStream byteArrayOS = new ByteArrayOutputStream();
    image.compress(compressFormat, quality, byteArrayOS);
    return Base64.encodeToString(byteArrayOS.toByteArray(), Base64.NO_WRAP);
  }
}
